local myname, ns = ...

local races = {
    [1] = "Human",
    [3] = "Dwarf",
    [4] = "NightElf",
    [11] = "Draenei",
    [22] = "Worgen",
    [7] = "Gnome",
    [24] = "Pandaren",
    [25] = "Pandaren",
    [26] = "Pandaren",
    [2] = "Orc",
    [5] = "Scourge",
    [10] = "BloodElf",
    [8] = "Troll",
    [6] = "Tauren",
    [9] = "Goblin",
    [52] = "Dracthyr",
    -- Allied!
    [27] = "Nightborne", -- "Nightborne",
    [28] = "HighmountainTauren", -- "HighmountainTauren",
    [29] = "VoidElf", -- "VoidElf",
    [30] = "LightforgedDraenei", -- "LightforgedDraenei",
    [31] = "ZandalariTroll",
    [32] = "KulTiran",
    [34] = "DarkIronDwarf",
    [35] = "Vulpera",
    [36] = "MagharOrc", -- "MagharOrc",
    [37] = "Mechagnome",
}
local fallback_races = {
    Nightborne = {"BloodElf", "NightElf"}, -- Nightborne -> male Blood Elf / female Night Elf
    MagharOrc = "Orc", -- Maghar -> Orc
    LightforgedDraenei = "Draenei", -- Lightforged -> Draenei
    KulTiran = "Human", -- Kul'Tiran -> Human
    HighmountainTauren = "Tauren", -- Highmountain -> Tauren
    VoidElf = "BloodElf", -- Void Elf -> Blood Elf
    Mechagnome = "Gnome", -- Mechagnome -> Gnome
    Vulpera = "Goblin", -- Vulpera -> Goblin
    ZandalariTroll = "Troll", -- Zandalari -> Troll
    DarkIronDwarf = "Dwarf", -- Dark Iron -> Dwarf
    Dracthyr = {"Draenei", "Human"}, -- Dracthyr -> male Draenei / female Human
}
local genders = {
    [0] = "Male",
    [1] = "Female",
}

local slots = {
    INVTYPE_BODY = "Shirt",
    INVTYPE_CHEST = "Shirt",
    INVTYPE_CLOAK = "Back",
    INVTYPE_FEET = "Feet",
    INVTYPE_HAND = "Hands",
    INVTYPE_HEAD = "Head",
    INVTYPE_LEGS = "Legs",
    INVTYPE_ROBE = "Robe",
    INVTYPE_SHOULDER = "Shoulder",
    -- INVTYPE_SHOULDER = "Shoulder-Alt",
    INVTYPE_TABARD = "Tabard",
    INVTYPE_WAIST = "Waist",
    INVTYPE_WRIST = "Wrist",
}
local item_slots = {
    INVTYPE_2HWEAPON = true,
    INVTYPE_WEAPON = true,
    INVTYPE_WEAPONMAINHAND = true,
    INVTYPE_WEAPONOFFHAND = true,
    INVTYPE_RANGED = true,
    INVTYPE_RANGEDRIGHT = true,
    INVTYPE_HOLDABLE = "Offhand",
    INVTYPE_SHIELD = "Shield",
}
local subclasses = {}
for k,v in pairs(Enum.ItemWeaponSubclass) do
    subclasses[v] = k
end
subclasses[Enum.ItemWeaponSubclass.Fishingpole] = "Staff"
subclasses[Enum.ItemWeaponSubclass.Generic] = "Sword1H"

local _, _, playerRaceID = UnitRace("player")
local playerSex
if UnitSex("player") == 2 then
    playerSex = "Male"
else
    playerSex = "Female"
end

local slots_to_cameraids, slot_override

-- Get a cameraid for Model_ApplyUICamera which will focus a DressUpModel on a specific item
-- itemid: number/string Anything that GetItemInfoInstant will accept
-- race: number raceid
-- gender: number genderid (0: male, 1: female)
function ns:GetCameraID(itemLinkOrID, raceID, genderID)
    local key, itemcamera
    local itemid, _, _, slot, _, class, subclass = GetItemInfoInstant(itemLinkOrID)
    if item_slots[slot] then
        itemcamera = true
        if item_slots[slot] == true then
            key = "Weapon-" .. subclasses[subclass] or subclasses["Generic"]
        else
            key = "Weapon-" .. item_slots[slot]
        end
    else
        local race = races[raceID or playerRaceID]
        local gender = genderID and genders[genderID] or playerSex
        if not raceID then
            -- alt form races need special handling:
            if race == 'Worgen' and select(2, C_PlayerInfo.GetAlternateFormInfo()) then
                race = 'Human'
            end
            if race == 'Dracthyr' and not select(2, C_PlayerInfo.GetAlternateFormInfo()) then
                gender = 'Male'
            end
        end
        key = ("%s-%s-%s"):format(race, gender, slot_override[itemid] or slots[slot] or "Default")
        if not slots_to_cameraids[key] and fallback_races[race] then
            local fallback = fallback_races[race]
            if type(fallback) == "table" then
                fallback = fallback[genderID == 0 and 1 or 2]
            end
            key = ("%s-%s-%s"):format(fallback, gender, slot_override[itemid] or slots[slot] or "Default")
        end
    end
    -- ns.Debug("GetCameraID", key, slots_to_cameraids[key], itemcamera)
    return slots_to_cameraids[key], itemcamera
end

-- https://wow.tools/dbc/?dbc=uicamera
-- Transmog-[race]-[gender]-[slot]
slots_to_cameraids = {
    -- the 1h/2h have names altered to fit with the Enum.ItemWeaponSubclass names
    ["Weapon-Axe1H"] = 242,
    ["Weapon-Mace1H"] = 244,
    ["Weapon-Sword1H"] = 238,
    ["Weapon-Axe2H"] = 243,
    ["Weapon-Mace2H"] = 245,
    ["Weapon-Sword2H"] = 239,
    ["Weapon-Bows"] = 251, -- Bow
    ["Weapon-Crossbow"] = 253,
    ["Weapon-Dagger"] = 241,
    ["Weapon-Unarmed"] = 248, -- FistWeapon
    ["Weapon-Warglaive"] = 624, -- Glaive
    ["Weapon-Gun"] = 252,
    ["Weapon-Polearm"] = 247,
    ["Weapon-Shield"] = 249,
    ["Weapon-Staff"] = 246,
    ["Weapon-Wand"] = 240,
    ["Weapon-Offhand"] = 250,
    --
    ["BloodElf-Female-Back"] = 467,
    ["BloodElf-Female-Back-Backpack"] = 1490,
    ["BloodElf-Female-Feet"] = 475,
    ["BloodElf-Female-Hands"] = 472,
    ["BloodElf-Female-Head"] = 465,
    ["BloodElf-Female-Legs"] = 474,
    ["BloodElf-Female-Robe"] = 468,
    ["BloodElf-Female-Shirt"] = 469,
    ["BloodElf-Female-Shoulder"] = 466,
    ["BloodElf-Female-Shoulder-Alt"] = 739,
    ["BloodElf-Female-Tabard"] = 470,
    ["BloodElf-Female-Waist"] = 473,
    ["BloodElf-Female-Wrist"] = 471,
    ["BloodElf-Male-Back"] = 456,
    ["BloodElf-Male-Back-Backpack"] = 1491,
    ["BloodElf-Male-Feet"] = 464,
    ["BloodElf-Male-Hands"] = 461,
    ["BloodElf-Male-Head"] = 454,
    ["BloodElf-Male-Legs"] = 463,
    ["BloodElf-Male-Robe"] = 457,
    ["BloodElf-Male-Shirt"] = 458,
    ["BloodElf-Male-Shoulder"] = 455,
    ["BloodElf-Male-Shoulder-Alt"] = 738,
    ["BloodElf-Male-Tabard"] = 459,
    ["BloodElf-Male-Waist"] = 462,
    ["BloodElf-Male-Wrist"] = 460,
    ["Dracthyr-Male-Back"] = 1706,
    ["Dracthyr-Male-Back-Backpack"] = 1699,
    ["Dracthyr-Male-Feet"] = 1705,
    ["Dracthyr-Male-Hands"] = 1708,
    ["Dracthyr-Male-Head"] = 1702,
    ["Dracthyr-Male-Legs"] = 1701,
    ["Dracthyr-Male-Robe"] = 1698,
    ["Dracthyr-Male-Shirt"] = 1709,
    ["Dracthyr-Male-Shoulder"] = 1704,
    ["Dracthyr-Male-Shoulder-Alt"] = 1703,
    ["Dracthyr-Male-Tabard"] = 1707,
    ["Dracthyr-Male-Waist"] = 1700,
    ["Dracthyr-Male-Wrist"] = 1711,
    ["Draenei-Female-Back"] = 345,
    ["Draenei-Female-Back-Backpack"] = 1492,
    ["Draenei-Female-Feet"] = 358,
    ["Draenei-Female-Hands"] = 352,
    ["Draenei-Female-Head"] = 342,
    ["Draenei-Female-Legs"] = 356,
    ["Draenei-Female-Robe"] = 347,
    ["Draenei-Female-Shirt"] = 348,
    ["Draenei-Female-Shoulder"] = 343,
    ["Draenei-Female-Shoulder-Alt"] = 730,
    ["Draenei-Female-Tabard"] = 349,
    ["Draenei-Female-Waist"] = 355,
    ["Draenei-Female-Wrist"] = 350,
    ["Draenei-Male-Back"] = 333,
    ["Draenei-Male-Back-Backpack"] = 1493,
    ["Draenei-Male-Chest"] = 677,
    ["Draenei-Male-Feet"] = 341,
    ["Draenei-Male-Hands"] = 338,
    ["Draenei-Male-Head"] = 331,
    ["Draenei-Male-Legs"] = 340,
    ["Draenei-Male-Robe"] = 334,
    ["Draenei-Male-Shirt"] = 335,
    ["Draenei-Male-Shoulder"] = 332,
    ["Draenei-Male-Shoulder-Alt"] = 729,
    ["Draenei-Male-Tabard"] = 336,
    ["Draenei-Male-Waist"] = 339,
    ["Draenei-Male-Wrist"] = 337,
    ["Dwarf-Female-Back"] = 376,
    ["Dwarf-Female-Back-Backpack"] = 1494,
    ["Dwarf-Female-Feet"] = 384,
    ["Dwarf-Female-Hands"] = 381,
    ["Dwarf-Female-Head"] = 374,
    ["Dwarf-Female-Legs"] = 383,
    ["Dwarf-Female-Robe"] = 377,
    ["Dwarf-Female-Shirt"] = 378,
    ["Dwarf-Female-Shoulder"] = 375,
    ["Dwarf-Female-Shoulder-Alt"] = 809,
    ["Dwarf-Female-Tabard"] = 379,
    ["Dwarf-Female-Waist"] = 382,
    ["Dwarf-Female-Wrist"] = 380,
    ["Dwarf-Male-Back"] = 365,
    ["Dwarf-Male-Back-Backpack"] = 1495,
    ["Dwarf-Male-Feet"] = 373,
    ["Dwarf-Male-Hands"] = 370,
    ["Dwarf-Male-Head"] = 363,
    ["Dwarf-Male-Legs"] = 372,
    ["Dwarf-Male-Robe"] = 366,
    ["Dwarf-Male-Shirt"] = 367,
    ["Dwarf-Male-Shoulder"] = 364,
    ["Dwarf-Male-Shoulder-Alt"] = 731,
    ["Dwarf-Male-Tabard"] = 368,
    ["Dwarf-Male-Waist"] = 371,
    ["Dwarf-Male-Wrist"] = 369,
    ["Gnome-Female-Back"] = 401,
    ["Gnome-Female-Back-Backpack"] = 1496,
    ["Gnome-Female-Feet"] = 409,
    ["Gnome-Female-Hands"] = 406,
    ["Gnome-Female-Head"] = 399,
    ["Gnome-Female-Legs"] = 408,
    ["Gnome-Female-Robe"] = 402,
    ["Gnome-Female-Shirt"] = 403,
    ["Gnome-Female-Shoulder"] = 400,
    ["Gnome-Female-Shoulder-Alt"] = 733,
    ["Gnome-Female-Tabard"] = 404,
    ["Gnome-Female-Waist"] = 407,
    ["Gnome-Female-Wrist"] = 405,
    ["Gnome-Male-Back"] = 387,
    ["Gnome-Male-Back-Backpack"] = 1497,
    ["Gnome-Male-Feet"] = 398,
    ["Gnome-Male-Hands"] = 395,
    ["Gnome-Male-Head"] = 385,
    ["Gnome-Male-Legs"] = 397,
    ["Gnome-Male-Robe"] = 389,
    ["Gnome-Male-Shirt"] = 390,
    ["Gnome-Male-Shoulder"] = 386,
    ["Gnome-Male-Shoulder-Alt"] = 732,
    ["Gnome-Male-Tabard"] = 393,
    ["Gnome-Male-Waist"] = 396,
    ["Gnome-Male-Wrist"] = 394,
    ["Goblin-Female-Back"] = 445,
    ["Goblin-Female-Back-Backpack"] = 1498,
    ["Goblin-Female-Feet"] = 453,
    ["Goblin-Female-Hands"] = 450,
    ["Goblin-Female-Head"] = 443,
    ["Goblin-Female-Legs"] = 452,
    ["Goblin-Female-Robe"] = 446,
    ["Goblin-Female-Shirt"] = 447,
    ["Goblin-Female-Shoulder"] = 444,
    ["Goblin-Female-Shoulder-Alt"] = 737,
    ["Goblin-Female-Tabard"] = 448,
    ["Goblin-Female-Waist"] = 451,
    ["Goblin-Female-Wrist"] = 449,
    ["Goblin-Male-Back"] = 434,
    ["Goblin-Male-Back-Backpack"] = 1499,
    ["Goblin-Male-Feet"] = 442,
    ["Goblin-Male-Hands"] = 439,
    ["Goblin-Male-Head"] = 432,
    ["Goblin-Male-Legs"] = 441,
    ["Goblin-Male-Robe"] = 435,
    ["Goblin-Male-Shirt"] = 436,
    ["Goblin-Male-Shoulder"] = 433,
    ["Goblin-Male-Shoulder-Alt"] = 736,
    ["Goblin-Male-Tabard"] = 437,
    ["Goblin-Male-Waist"] = 440,
    ["Goblin-Male-Wrist"] = 438,
    ["HighmountainTauren-Female-Back"] = 1207,
    ["HighmountainTauren-Female-Back-Backpack"] = 1500,
    ["HighmountainTauren-Female-Feet"] = 1206,
    ["HighmountainTauren-Female-Hands"] = 1205,
    ["HighmountainTauren-Female-Head"] = 1204,
    ["HighmountainTauren-Female-Legs"] = 1203,
    ["HighmountainTauren-Female-Robe"] = 1202,
    ["HighmountainTauren-Female-Shirt"] = 1201,
    ["HighmountainTauren-Female-Shoulder"] = 1200,
    ["HighmountainTauren-Female-Shoulder-Alt"] = 1199,
    ["HighmountainTauren-Female-Tabard"] = 1198,
    ["HighmountainTauren-Female-Waist"] = 1197,
    ["HighmountainTauren-Female-Wrist"] = 1196,
    ["HighmountainTauren-Male-Back"] = 1195,
    ["HighmountainTauren-Male-Back-Backpack"] = 1501,
    ["HighmountainTauren-Male-Feet"] = 1194,
    ["HighmountainTauren-Male-Hands"] = 1193,
    ["HighmountainTauren-Male-Head"] = 1192,
    ["HighmountainTauren-Male-Legs"] = 1191,
    ["HighmountainTauren-Male-Robe"] = 1190,
    ["HighmountainTauren-Male-Shirt"] = 1189,
    ["HighmountainTauren-Male-Shoulder"] = 1188,
    ["HighmountainTauren-Male-Shoulder-Alt"] = 1187,
    ["HighmountainTauren-Male-Tabard"] = 1186,
    ["HighmountainTauren-Male-Waist"] = 1185,
    ["HighmountainTauren-Male-Wrist"] = 1184,
    ["Human-Female-Back"] = 276,
    ["Human-Female-Back-Backpack"] = 1502,
    ["Human-Female-Feet"] = 284,
    ["Human-Female-Hands"] = 281,
    ["Human-Female-Head"] = 274,
    ["Human-Female-Legs"] = 283,
    ["Human-Female-Robe"] = 277,
    ["Human-Female-Shirt"] = 278,
    ["Human-Female-Shoulder"] = 275,
    ["Human-Female-Shoulder-Alt"] = 724,
    ["Human-Female-Tabard"] = 279,
    ["Human-Female-Waist"] = 282,
    ["Human-Female-Wrist"] = 280,
    ["Human-Male-Back"] = 235,
    ["Human-Male-Back-Backpack"] = 1503,
    ["Human-Male-Chest"] = 674,
    ["Human-Male-Feet"] = 227,
    ["Human-Male-Hands"] = 226,
    ["Human-Male-Head"] = 236,
    ["Human-Male-Legs"] = 228,
    ["Human-Male-Robe"] = 225,
    ["Human-Male-Shirt"] = 229,
    ["Human-Male-Shoulder"] = 221,
    ["Human-Male-Shoulder-Alt"] = 723,
    ["Human-Male-Tabard"] = 230,
    ["Human-Male-Waist"] = 234,
    ["Human-Male-Wrist"] = 237,
    ["KulTiran-Female-Back"] = 1406,
    ["KulTiran-Female-Back-Backpack"] = 1504,
    ["KulTiran-Female-Chest"] = 1407,
    ["KulTiran-Female-Feet"] = 1413,
    ["KulTiran-Female-Hands"] = 1410,
    ["KulTiran-Female-Head"] = 1403,
    ["KulTiran-Female-Legs"] = 1412,
    ["KulTiran-Female-Robe"] = 1414,
    ["KulTiran-Female-Shoulder"] = 1404,
    ["KulTiran-Female-Shoulder-Alt"] = 1405,
    ["KulTiran-Female-Tabard"] = 1408,
    ["KulTiran-Female-Waist"] = 1411,
    ["KulTiran-Female-Wrist"] = 1409,
    ["KulTiran-Male-Back"] = 1393,
    ["KulTiran-Male-Back-Backpack"] = 1505,
    ["KulTiran-Male-Chest"] = 1394,
    ["KulTiran-Male-Feet"] = 1400,
    ["KulTiran-Male-Hands"] = 1397,
    ["KulTiran-Male-Head"] = 1383,
    ["KulTiran-Male-Legs"] = 1399,
    ["KulTiran-Male-Robe"] = 1401,
    ["KulTiran-Male-Shoulder"] = 1384,
    ["KulTiran-Male-Shoulder-Alt"] = 1402,
    ["KulTiran-Male-Tabard"] = 1395,
    ["KulTiran-Male-Waist"] = 1398,
    ["KulTiran-Male-Wrist"] = 1396,
    ["LightforgedDraenei-Female-Back"] = 1151,
    ["LightforgedDraenei-Female-Back-Backpack"] = 1506,
    ["LightforgedDraenei-Female-Feet"] = 1150,
    ["LightforgedDraenei-Female-Hands"] = 1149,
    ["LightforgedDraenei-Female-Head"] = 1148,
    ["LightforgedDraenei-Female-Legs"] = 1147,
    ["LightforgedDraenei-Female-Robe"] = 1146,
    ["LightforgedDraenei-Female-Shirt"] = 1145,
    ["LightforgedDraenei-Female-Shoulder"] = 1144,
    ["LightforgedDraenei-Female-Shoulder-Alt"] = 1143,
    ["LightforgedDraenei-Female-Tabard"] = 1142,
    ["LightforgedDraenei-Female-Waist"] = 1141,
    ["LightforgedDraenei-Female-Wrist"] = 1140,
    ["LightforgedDraenei-Male-Back"] = 1139,
    ["LightforgedDraenei-Male-Back-Backpack"] = 1507,
    ["LightforgedDraenei-Male-Chest"] = 1138,
    ["LightforgedDraenei-Male-Feet"] = 1137,
    ["LightforgedDraenei-Male-Hands"] = 1136,
    ["LightforgedDraenei-Male-Head"] = 1135,
    ["LightforgedDraenei-Male-Legs"] = 1134,
    ["LightforgedDraenei-Male-Robe"] = 1133,
    ["LightforgedDraenei-Male-Shirt"] = 1132,
    ["LightforgedDraenei-Male-Shoulder"] = 1131,
    ["LightforgedDraenei-Male-Shoulder-Alt"] = 1130,
    ["LightforgedDraenei-Male-Tabard"] = 1129,
    ["LightforgedDraenei-Male-Waist"] = 1128,
    ["LightforgedDraenei-Male-Wrist"] = 1127,
    ["MagharOrc-Male-Back"] = 1361,
    ["MagharOrc-Male-Back-Backpack"] = 1508,
    ["MagharOrc-Male-Feet"] = 1369,
    ["MagharOrc-Male-Hands"] = 1366,
    ["MagharOrc-Male-Head"] = 1359,
    ["MagharOrc-Male-Legs"] = 1368,
    ["MagharOrc-Male-Robe"] = 1362,
    ["MagharOrc-Male-Shirt"] = 1363,
    ["MagharOrc-Male-Shoulder"] = 1360,
    ["MagharOrc-Male-Tabard"] = 1364,
    ["MagharOrc-Male-Waist"] = 1367,
    ["MagharOrc-Male-Wrist"] = 1365,
    ["Mechagnome-Female-Hands"] = 1534,
    ["Mechagnome-Male-Hands"] = 1533,
    ["Nightborne-Female-Back"] = 1099,
    ["Nightborne-Female-Back-Backpack"] = 1509,
    ["Nightborne-Female-Feet"] = 1106,
    ["Nightborne-Female-Hands"] = 1103,
    ["Nightborne-Female-Head"] = 1096,
    ["Nightborne-Female-Legs"] = 1105,
    ["Nightborne-Female-Robe"] = 1107,
    ["Nightborne-Female-Shirt"] = 1100,
    ["Nightborne-Female-Shoulder"] = 1097,
    ["Nightborne-Female-Shoulder-Alt"] = 1098,
    ["Nightborne-Female-Tabard"] = 1101,
    ["Nightborne-Female-Waist"] = 1104,
    ["Nightborne-Female-Wrist"] = 1102,
    ["Nightborne-Male-Back"] = 1183,
    ["Nightborne-Male-Back-Backpack"] = 1510,
    ["Nightborne-Male-Feet"] = 1095,
    ["Nightborne-Male-Hands"] = 1182,
    ["Nightborne-Male-Head"] = 1090,
    ["Nightborne-Male-Legs"] = 1181,
    ["Nightborne-Male-Robe"] = 1091,
    ["Nightborne-Male-Shirt"] = 1092,
    ["Nightborne-Male-Shoulder"] = 1180,
    ["Nightborne-Male-Shoulder-Alt"] = 1179,
    ["Nightborne-Male-Tabard"] = 1178,
    ["Nightborne-Male-Waist"] = 1177,
    ["Nightborne-Male-Wrist"] = 1176,
    ["NightElf-Female-Back"] = 423,
    ["NightElf-Female-Back-Backpack"] = 1511,
    ["NightElf-Female-Feet"] = 431,
    ["NightElf-Female-Hands"] = 428,
    ["NightElf-Female-Head"] = 421,
    ["NightElf-Female-Legs"] = 430,
    ["NightElf-Female-Robe"] = 424,
    ["NightElf-Female-Shirt"] = 425,
    ["NightElf-Female-Shoulder"] = 422,
    ["NightElf-Female-Shoulder-Alt"] = 735,
    ["NightElf-Female-Tabard"] = 426,
    ["NightElf-Female-Waist"] = 429,
    ["NightElf-Female-Wrist"] = 427,
    ["NightElf-Male-Back"] = 412,
    ["NightElf-Male-Back-Backpack"] = 1512,
    ["NightElf-Male-Feet"] = 420,
    ["NightElf-Male-Hands"] = 417,
    ["NightElf-Male-Head"] = 410,
    ["NightElf-Male-Legs"] = 419,
    ["NightElf-Male-Robe"] = 413,
    ["NightElf-Male-Shirt"] = 414,
    ["NightElf-Male-Shoulder"] = 411,
    ["NightElf-Male-Shoulder-Alt"] = 734,
    ["NightElf-Male-Tabard"] = 415,
    ["NightElf-Male-Waist"] = 418,
    ["NightElf-Male-Wrist"] = 416,
    ["Orc-Female-Back"] = 489,
    ["Orc-Female-Back-Backpack"] = 1513,
    ["Orc-Female-Feet"] = 497,
    ["Orc-Female-Hands"] = 494,
    ["Orc-Female-Head"] = 487,
    ["Orc-Female-Legs"] = 496,
    ["Orc-Female-Robe"] = 490,
    ["Orc-Female-Shirt"] = 491,
    ["Orc-Female-Shoulder"] = 488,
    ["Orc-Female-Shoulder-Alt"] = 741,
    ["Orc-Female-Tabard"] = 492,
    ["Orc-Female-Waist"] = 495,
    ["Orc-Female-Wrist"] = 493,
    ["Orc-Male-Back"] = 478,
    ["Orc-Male-Back-Backpack"] = 1514,
    ["Orc-Male-Feet"] = 486,
    ["Orc-Male-Hands"] = 483,
    ["Orc-Male-Head"] = 476,
    ["Orc-Male-Legs"] = 485,
    ["Orc-Male-Robe"] = 479,
    ["Orc-Male-Shirt"] = 480,
    ["Orc-Male-Shoulder"] = 477,
    ["Orc-Male-Shoulder-Alt"] = 740,
    ["Orc-Male-Tabard"] = 481,
    ["Orc-Male-Waist"] = 484,
    ["Orc-Male-Wrist"] = 482,
    ["Pandaren-Female-Back"] = 300,
    ["Pandaren-Female-Back-Backpack"] = 1515,
    ["Pandaren-Female-Chest"] = 676,
    ["Pandaren-Female-Feet"] = 308,
    ["Pandaren-Female-Hands"] = 305,
    ["Pandaren-Female-Head"] = 298,
    ["Pandaren-Female-Legs"] = 307,
    ["Pandaren-Female-Robe"] = 301,
    ["Pandaren-Female-Shirt"] = 302,
    ["Pandaren-Female-Shoulder"] = 299,
    ["Pandaren-Female-Shoulder-Alt"] = 726,
    ["Pandaren-Female-Tabard"] = 303,
    ["Pandaren-Female-Waist"] = 306,
    ["Pandaren-Female-Wrist"] = 304,
    ["Pandaren-Male-Back"] = 287,
    ["Pandaren-Male-Back-Backpack"] = 1516,
    ["Pandaren-Male-Chest"] = 675,
    ["Pandaren-Male-Feet"] = 295,
    ["Pandaren-Male-Hands"] = 292,
    ["Pandaren-Male-Head"] = 285,
    ["Pandaren-Male-Legs"] = 294,
    ["Pandaren-Male-Robe"] = 288,
    ["Pandaren-Male-Shirt"] = 289,
    ["Pandaren-Male-Shoulder"] = 286,
    ["Pandaren-Male-Shoulder-Alt"] = 725,
    ["Pandaren-Male-Tabard"] = 290,
    ["Pandaren-Male-Waist"] = 293,
    ["Pandaren-Male-Wrist"] = 291,
    ["Tauren-Female-Back"] = 511,
    ["Tauren-Female-Back-Backpack"] = 1517,
    ["Tauren-Female-Feet"] = 519,
    ["Tauren-Female-Hands"] = 516,
    ["Tauren-Female-Head"] = 509,
    ["Tauren-Female-Legs"] = 518,
    ["Tauren-Female-Robe"] = 512,
    ["Tauren-Female-Shirt"] = 513,
    ["Tauren-Female-Shoulder"] = 510,
    ["Tauren-Female-Shoulder-Alt"] = 743,
    ["Tauren-Female-Tabard"] = 514,
    ["Tauren-Female-Waist"] = 517,
    ["Tauren-Female-Wrist"] = 515,
    ["Tauren-Male-Back"] = 500,
    ["Tauren-Male-Back-Backpack"] = 1518,
    ["Tauren-Male-Feet"] = 508,
    ["Tauren-Male-Hands"] = 505,
    ["Tauren-Male-Head"] = 498,
    ["Tauren-Male-Legs"] = 507,
    ["Tauren-Male-Robe"] = 501,
    ["Tauren-Male-Shirt"] = 502,
    ["Tauren-Male-Shoulder"] = 499,
    ["Tauren-Male-Shoulder-Alt"] = 742,
    ["Tauren-Male-Tabard"] = 503,
    ["Tauren-Male-Waist"] = 506,
    ["Tauren-Male-Wrist"] = 504,
    ["Troll-Female-Back"] = 533,
    ["Troll-Female-Back-Backpack"] = 1519,
    ["Troll-Female-Feet"] = 541,
    ["Troll-Female-Hands"] = 538,
    ["Troll-Female-Head"] = 531,
    ["Troll-Female-Legs"] = 540,
    ["Troll-Female-Robe"] = 534,
    ["Troll-Female-Shirt"] = 535,
    ["Troll-Female-Shoulder"] = 532,
    ["Troll-Female-Shoulder-Alt"] = 745,
    ["Troll-Female-Tabard"] = 536,
    ["Troll-Female-Waist"] = 539,
    ["Troll-Female-Wrist"] = 537,
    ["Troll-Male-Back"] = 522,
    ["Troll-Male-Back-Backpack"] = 1520,
    ["Troll-Male-Chest"] = 689,
    ["Troll-Male-Feet"] = 530,
    ["Troll-Male-Hands"] = 527,
    ["Troll-Male-Head"] = 520,
    ["Troll-Male-Legs"] = 529,
    ["Troll-Male-Robe"] = 523,
    ["Troll-Male-Shirt"] = 524,
    ["Troll-Male-Shoulder"] = 521,
    ["Troll-Male-Shoulder-Alt"] = 744,
    ["Troll-Male-Tabard"] = 525,
    ["Troll-Male-Waist"] = 528,
    ["Troll-Male-Wrist"] = 526,
    ["Undead-Female-Back"] = 555,
    ["Undead-Female-Back-Backpack"] = 1521,
    ["Undead-Female-Feet"] = 563,
    ["Undead-Female-Hands"] = 560,
    ["Undead-Female-Head"] = 553,
    ["Undead-Female-Legs"] = 562,
    ["Undead-Female-Robe"] = 556,
    ["Undead-Female-Shirt"] = 557,
    ["Undead-Female-Shoulder"] = 554,
    ["Undead-Female-Shoulder-Alt"] = 747,
    ["Undead-Female-Tabard"] = 558,
    ["Undead-Female-Waist"] = 561,
    ["Undead-Female-Wrist"] = 559,
    ["Undead-Male-Back"] = 544,
    ["Undead-Male-Back-Backpack"] = 1522,
    ["Undead-Male-Chest"] = 690,
    ["Undead-Male-Feet"] = 552,
    ["Undead-Male-Hands"] = 549,
    ["Undead-Male-Head"] = 542,
    ["Undead-Male-Legs"] = 551,
    ["Undead-Male-Robe"] = 545,
    ["Undead-Male-Shirt"] = 546,
    ["Undead-Male-Shoulder"] = 543,
    ["Undead-Male-Shoulder-Alt"] = 746,
    ["Undead-Male-Tabard"] = 547,
    ["Undead-Male-Waist"] = 550,
    ["Undead-Male-Wrist"] = 548,
    ["VoidElf-Female-Back"] = 1175,
    ["VoidElf-Female-Back-Backpack"] = 1523,
    ["VoidElf-Female-Feet"] = 1174,
    ["VoidElf-Female-Hands"] = 1173,
    ["VoidElf-Female-Head"] = 1172,
    ["VoidElf-Female-Legs"] = 1171,
    ["VoidElf-Female-Robe"] = 1170,
    ["VoidElf-Female-Shirt"] = 1169,
    ["VoidElf-Female-Shoulder"] = 1168,
    ["VoidElf-Female-Shoulder-Alt"] = 1167,
    ["VoidElf-Female-Tabard"] = 1166,
    ["VoidElf-Female-Waist"] = 1165,
    ["VoidElf-Female-Wrist"] = 1164,
    ["VoidElf-Male-Back"] = 1163,
    ["VoidElf-Male-Back-Backpack"] = 1524,
    ["VoidElf-Male-Feet"] = 1162,
    ["VoidElf-Male-Hands"] = 1161,
    ["VoidElf-Male-Head"] = 1160,
    ["VoidElf-Male-Legs"] = 1159,
    ["VoidElf-Male-Robe"] = 1158,
    ["VoidElf-Male-Shirt"] = 1157,
    ["VoidElf-Male-Shoulder"] = 1156,
    ["VoidElf-Male-Shoulder-Alt"] = 1155,
    ["VoidElf-Male-Tabard"] = 1154,
    ["VoidElf-Male-Waist"] = 1153,
    ["VoidElf-Male-Wrist"] = 1152,
    ["Vulpera-Female-Back"] = 1477,
    ["Vulpera-Female-Back-Backpack"] = 1525,
    ["Vulpera-Female-Feet"] = 1484,
    ["Vulpera-Female-Hands"] = 1481,
    ["Vulpera-Female-Head"] = 1474,
    ["Vulpera-Female-Legs"] = 1483,
    ["Vulpera-Female-Robe"] = 1485,
    ["Vulpera-Female-Shirt"] = 1478,
    ["Vulpera-Female-Shoulder"] = 1475,
    ["Vulpera-Female-Shoulder-Alt"] = 1476,
    ["Vulpera-Female-Tabard"] = 1479,
    ["Vulpera-Female-Waist"] = 1482,
    ["Vulpera-Female-Wrist"] = 1480,
    ["Vulpera-Male-Back"] = 1466,
    ["Vulpera-Male-Back-Backpack"] = 1526,
    ["Vulpera-Male-Feet"] = 1473,
    ["Vulpera-Male-Hands"] = 1470,
    ["Vulpera-Male-Head"] = 1463,
    ["Vulpera-Male-Legs"] = 1472,
    ["Vulpera-Male-Shirt"] = 1467,
    ["Vulpera-Male-Shoulder"] = 1464,
    ["Vulpera-Male-Shoulder-Alt"] = 1465,
    ["Vulpera-Male-Tabard"] = 1468,
    ["Vulpera-Male-Waist"] = 1471,
    ["Vulpera-Male-Wrist"] = 1469,
    ["Worgen-Female-Back"] = 322,
    ["Worgen-Female-Back-Backpack"] = 1527,
    ["Worgen-Female-Feet"] = 330,
    ["Worgen-Female-Hands"] = 327,
    ["Worgen-Female-Head"] = 320,
    ["Worgen-Female-Legs"] = 329,
    ["Worgen-Female-Robe"] = 323,
    ["Worgen-Female-Shirt"] = 324,
    ["Worgen-Female-Shoulder"] = 321,
    ["Worgen-Female-Shoulder-Alt"] = 728,
    ["Worgen-Female-Tabard"] = 325,
    ["Worgen-Female-Waist"] = 328,
    ["Worgen-Female-Wrist"] = 326,
    ["Worgen-Male-Back"] = 311,
    ["Worgen-Male-Back-Backpack"] = 1528,
    ["Worgen-Male-Feet"] = 319,
    ["Worgen-Male-Hands"] = 316,
    ["Worgen-Male-Head"] = 309,
    ["Worgen-Male-Legs"] = 318,
    ["Worgen-Male-Robe"] = 312,
    ["Worgen-Male-Shirt"] = 313,
    ["Worgen-Male-Shoulder"] = 310,
    ["Worgen-Male-Shoulder-Alt"] = 727,
    ["Worgen-Male-Tabard"] = 314,
    ["Worgen-Male-Waist"] = 317,
    ["Worgen-Male-Wrist"] = 315,
    ["ZandalariTroll-Female-Back"] = 1430,
    ["ZandalariTroll-Female-Back-Backpack"] = 1529,
    ["ZandalariTroll-Female-Chest"] = 1431,
    ["ZandalariTroll-Female-Feet"] = 1437,
    ["ZandalariTroll-Female-Hands"] = 1434,
    ["ZandalariTroll-Female-Head"] = 1427,
    ["ZandalariTroll-Female-Legs"] = 1436,
    ["ZandalariTroll-Female-Robe"] = 1438,
    ["ZandalariTroll-Female-Shoulder"] = 1428,
    ["ZandalariTroll-Female-Shoulder-Alt"] = 1429,
    ["ZandalariTroll-Female-Tabard"] = 1432,
    ["ZandalariTroll-Female-Waist"] = 1435,
    ["ZandalariTroll-Female-Wrist"] = 1433,
    ["ZandalariTroll-Male-Back"] = 1418,
    ["ZandalariTroll-Male-Back-Backpack"] = 1530,
    ["ZandalariTroll-Male-Chest"] = 1419,
    ["ZandalariTroll-Male-Feet"] = 1425,
    ["ZandalariTroll-Male-Hands"] = 1422,
    ["ZandalariTroll-Male-Head"] = 1415,
    ["ZandalariTroll-Male-Legs"] = 1424,
    ["ZandalariTroll-Male-Robe"] = 1426,
    ["ZandalariTroll-Male-Shoulder"] = 1416,
    ["ZandalariTroll-Male-Shoulder-Alt"] = 1417,
    ["ZandalariTroll-Male-Tabard"] = 1420,
    ["ZandalariTroll-Male-Waist"] = 1423,
    ["ZandalariTroll-Male-Wrist"] = 1421,
}

--[[
Get these in-game with:

local visuals = C_TransmogCollection.GetCategoryAppearances(Enum.TransmogCollectionType.Back + 1)
for _, visual in ipairs(visuals) do
    local cameraID = C_TransmogCollection.GetAppearanceCameraID(visual.visualID)
    if cameraID == 1492 then
        -- draenei-female-back-backpack
        local sources = C_TransmogCollection.GetAppearanceSources(visual.visualID)
        print(string.format("[%d] = \"Back-Backpack\", -- %s", sources[1].itemID, sources[1].name))
    end
end
]]
slot_override = {
    -- Cloth
    -- appearance 21971
    [106545] = "Shoulder-Alt", -- Orunai Shoulderpads
    [106578] = "Shoulder-Alt", -- Gordunni Shoulderpads
    [112610] = "Shoulder-Alt", -- Steamburst Mantle
    [114271] = "Shoulder-Alt", -- Firefly mantle
    -- appearance 21620
    [106479] = "Shoulder-Alt", -- Iyun Shoulderpads
    [106512] = "Shoulder-Alt", -- Mandragoran Shoulderpads
    [107317] = "Shoulder-Alt", -- Karabor Sage Mantle
    [112086] = "Shoulder-Alt", -- Windburnt Pauldrons
    [106162] = "Shoulder-Alt", -- Frostwolf Wind-Talker Mantle
    -- appearance 21962
    [106413] = "Shoulder-Alt", -- Lunarglow Shoulderpads
    [106446] = "Shoulder-Alt", -- Anchorite Shoulderpads
    [112531] = "Shoulder-Alt", -- Auchenai Keeper Mantle
    -- Leather
    -- [] = "Shoulder-Alt", --
    -- Mail
    [7718] = "Shoulder-Alt", -- Herod's Shoulder
    [122356] = "Shoulder-Alt", -- Champion Herod's Shoulder
    [88271] = "Shoulder-Alt", -- Harlan's Shoulders
    -- Plate
    [140617] = "Shoulder-Alt", -- Rakeesh's Pauldron
    -- backpacks:
    [174361] = "Back-Backpack", -- Black Dragonscale Backpack
    [180939] = "Back-Backpack", -- Mantle of the Forgemaster's Dark Blades
    [180940] = "Back-Backpack", -- Ebony Crypt Keeper's Mantle
    [180941] = "Back-Backpack", -- Kael's Dark Sinstone Chain
    [181286] = "Back-Backpack", -- Halo of the Selfless
    [181287] = "Back-Backpack", -- Halo of the Reverent
    [181288] = "Back-Backpack", -- Halo of the Harmonious
    [181289] = "Back-Backpack", -- Halo of the Discordant
    [181290] = "Back-Backpack", -- Harmonious Sigil of the Archon
    [181291] = "Back-Backpack", -- Selfless Sigil of the Archon
    [181292] = "Back-Backpack", -- Discordant Sigil of the Archon
    [181293] = "Back-Backpack", -- Reverent Sigil of the Archon
    [181294] = "Back-Backpack", -- Harmonious Wings of the Ascended
    [181295] = "Back-Backpack", -- Selfless Wings of the Ascended
    [181296] = "Back-Backpack", -- Discordant Wings of the Ascended
    [181297] = "Back-Backpack", -- Reverent Wings of the Ascended
    [181301] = "Back-Backpack", -- Faewoven Branches
    [181302] = "Back-Backpack", -- Spirit Tender's Branches
    [181303] = "Back-Backpack", -- Night Courtier's Branches
    [181304] = "Back-Backpack", -- Winterwoven Branches
    [181305] = "Back-Backpack", -- Faewoven Bulb
    [181306] = "Back-Backpack", -- Spirit Tender's Bulb
    [181308] = "Back-Backpack", -- Winterwoven Bulb
    [181309] = "Back-Backpack", -- Faewoven Pack
    [181310] = "Back-Backpack", -- Spirit Tender's Pack
    [181312] = "Back-Backpack", -- Winterwoven Pack
    [181800] = "Back-Backpack", -- Standard of the Blackhound Warband
    [181801] = "Back-Backpack", -- Standard of the Necrolords
    [181802] = "Back-Backpack", -- Standard of Death's Chosen
    [181803] = "Back-Backpack", -- Bladesworn Battle Standard
    [181804] = "Back-Backpack", -- Trophy of the Reborn Bonelord
    [181805] = "Back-Backpack", -- Osteowings of the Necrolords
    [181806] = "Back-Backpack", -- Regrown Osteowings
    [181807] = "Back-Backpack", -- Barbarous Osteowings
    [181808] = "Back-Backpack", -- Death Fetish
    [181809] = "Back-Backpack", -- Tomalin's Seasoning Crystal
    [181810] = "Back-Backpack", -- Phylactery of the Dead Conniver
    [181811] = "Back-Backpack", -- Beckoner's Shadowy Crystal
    [183705] = "Back-Backpack", -- Mantle of Crimson Blades
    [183706] = "Back-Backpack", -- Mantle of Court Blades
    [183707] = "Back-Backpack", -- Mantle of Burnished Blades
    [183708] = "Back-Backpack", -- Glittering Gold Sinstone Chain
    [183709] = "Back-Backpack", -- Bronze-Bound Sinstone
    [183710] = "Back-Backpack", -- Burnished Sinstone Chain
    [183711] = "Back-Backpack", -- Burnished Crypt Keeper's Mantle
    [183712] = "Back-Backpack", -- Gleaming Crypt Keeper's Mantle
    [183713] = "Back-Backpack", -- Kassir's Crypt Mantle
    [184154] = "Back-Backpack", -- Grungy Containment Pack
    [184156] = "Back-Backpack", -- Pristine Containment Pack
}
